import os
import pkgutil
import subprocess
import tempfile

class export(object):
    def __init__(self, path, branch):
        self.path = path
        self.branch = branch
        self.proc = None

    def __enter__(self):
        with tempfile.NamedTemporaryFile(delete=False) as f:
            self.marks_path = f.name
            f.write(pkgutil.get_data('data', 'marks.bzr'))
        return self

    def __exit__(self, *args):
        os.remove(self.marks_path)

    def __iter__(self):
        self.proc = subprocess.Popen([
            'bzr', 'fast-export',
                '--plain',
                '-b', self.branch,
                '--import-marks', self.marks_path,
                self.path,
        ], stdout=subprocess.PIPE)

        stream = self.proc.stdout
        while True:
            line = stream.readline()
            if not line:
                return

            yield line

            if line.startswith('data '):
                size = int(line[len('data '):].strip())
                # yield a block of raw data as a line
                yield stream.read(size)
                # may be a newline here
                follow = stream.readline()
                # if it's *not* just a newline, yield it, it's the next line
                if follow and follow != '\n':
                    yield follow
